/*
 * Copyright 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "strl.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/* size_t */
#include <stddef.h>

#define GETDELIM_CHUNKSIZE (64)

ssize_t getdelim(char **lineptr, size_t *n, int delim, FILE *stream)
{
  ssize_t linelen;
  char *newlineptr;
  char c;

  if (!lineptr || !n || !stream)
    {
      errno = EINVAL;
      return -1;
    }

  if (!*lineptr)
    {
      *n = GETDELIM_CHUNKSIZE;
      *lineptr = malloc(*n);
    }
  if (!*lineptr)
    return -1;

  linelen = 0;
  while (1)
    {
      /*
       * Ensure adequate length. This has to be matched to the
       * subsequent for loop.
       */
      if (linelen + 2 > *n)
	{
	  *n += GETDELIM_CHUNKSIZE;
	  newlineptr = realloc(*lineptr, *n);
	  if (!newlineptr)
	    {
	      *n -= GETDELIM_CHUNKSIZE;
	      return -1;
	    }
	  *lineptr = newlineptr;
	}

      /*
       * Ensure we have space for two new chars: the one we read and
       * the terminating NUL.
       */
      errno = 0;
      for (; linelen < *n - 1; linelen ++)
	{
	  c = fgetc(stream);
	  if (c == EOF)
	    {
	      if (errno)
		{
		  /*
		   * Should we just set EINVAL for everything? We're
		   * supposed to set EINVAL for a bad stream
		   * descriptor... and I assume/hope that EBADF fits
		   * the description... --binki
		   */
		  if (errno == EBADF)
		    errno = EINVAL;
		  return -1;
		}
	      if (feof(stream))
		if (linelen)
		  {
		    /* we hit EOF _instead_ of a delimiter */
		    (*lineptr)[linelen] = '\0';
		    return linelen;
		  }
		else
		  /* since we've read no data, time to tell the caller it's EOF */
		  return -1;
	    }
	  (*lineptr)[linelen] = c;
	  if (c == delim)
	    {
	      linelen ++;
	      /* our for loop ensures that we have this second char to put the '\0' into */
	      (*lineptr)[linelen] = '\0';
	      return linelen;
	    }
	}
    }
}
