/*
 * Copyright 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include <check.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

#include "suite_asprintf.h"

#include "suite_getdelim.h"
#include "suite_getline.h"

#include "suite_strlcat.h"
#include "suite_strlcpy.h"
#include "suite_strnlen.h"

#include "suite_strndup.h"

#include "suite_wcslcat.h"
#include "suite_wcslcpy.h"
#include "suite_wcsnlen.h"

/**
 * \brief
 *   Use libcheck to run all of our testsuites.
 */
int main(int argc, char *argv[])
{
  int number_failed;
  size_t test_num;

  Suite *s;
  SRunner *sr;

  Suite *(*suites[])() =
    {
      &asprintf_suite,

      &getdelim_suite,
      &getline_suite,

      &strlcat_suite,
      &strlcpy_suite,
      &strnlen_suite,

      strndup_suite,

      &wcslcat_suite,
      &wcslcpy_suite,
      &wcsnlen_suite,

      NULL
    };

  number_failed = 0;

  for (test_num = 0; suites[test_num]; test_num ++)
    {
      s = (*suites[test_num])();
      sr = srunner_create(s);

      srunner_run_all(sr, CK_NORMAL);
      number_failed += srunner_ntests_failed(sr);
      srunner_free(sr);
    }

  if (number_failed)
    {
      fprintf(stderr, "total tests failed: %d\n", number_failed);
      return EXIT_FAILURE;
    }

  return EXIT_SUCCESS;
}
