/*
 * Copyright 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "suite_asprintf.h"

#include <check.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strl.h>

START_TEST (asprintf_general)
{
  char *s;
  char s_n[32];
  int len;

  /* Vanilla */
  snprintf(s_n, sizeof(s_n), "This is an exampl%d string %s.", 3, "blah");
  fail_unless(!strcmp(s_n, "This is an exampl3 string blah."),
	      "snprintf() does not behave as expected, got unexpected string: ``%s''.", s_n);

  len = asprintf(&s, "This is an exampl%d string %s.", 3, "blah");
  fail_unless(len == sizeof(s_n) - 1,
	      "Return value %d does not match expected value %d.",
	      len, sizeof(s_n) - 1);
  fail_unless(!strcmp(s, s_n),
	      "String produced by asprintf() (``%s'') does not match the string with the same format from snprintf() (``%s'').",
	      s, s_n);
  free(s);
}
END_TEST

START_TEST(asprintf_empty)
{
  char *s;
  int len;

  len = asprintf(&s, "");
  fail_unless(len == 0, "Return value %d does not match expected value of 0.", len);
  fail_unless(!strcmp(s, ""),
	      "String produced by asprintf() (``%s'') does not match the expected string ``''.",
	      s);
  free(s);
}
END_TEST

Suite *asprintf_suite()
{
  Suite *s;
  TCase *tc;

  s = suite_create("asprintf");

  tc = tcase_create("general");
  tcase_add_test(tc, asprintf_general);
  suite_add_tcase(s, tc);

  tc = tcase_create("empty");
  tcase_add_test(tc, asprintf_empty);
  suite_add_tcase(s, tc);

  return s;
}
