/*
 * Copyright 2011 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "suite_strndup.h"

#include <check.h>
#include <stdlib.h>
#include <string.h>
#include <strl.h>

START_TEST (strndup_normal)
{
  char *c;

  /* A string which is known to be shorter than 25 characters long. */
  const char *str = "My string to be duped.";

  c = strndup(str, 25);
  fail_if(!c, "strndup() returned NULL and I'm quite sure there is adequate memory.");
  fail_unless(strlen(c) == strlen(str),
	      "Expected output string to be as long as the input string. strlen(c) == %zu, strlen(str) == %zu.",
	      strlen(c), strlen(str));
  fail_unless(!strcmp(c, str),
	      "Expected output string to be equal to the input string. Instead, they differ.\n"
	      "\tin: %s\n"
	      "\tout: %s",
	      str, c);

  free(c);
}
END_TEST

START_TEST (strndup_longstring)
{
  char *c;

  /* A string which is known to be longer then 25 chars. */
  const char *str = "My my my! This is a terrrrrrrrrrrribly loooooooonnnnng string! :-0.";

  c = strndup(str, 25);
  fail_if(!c, "strndup() returned NULL and I'm quite sure there is adequate memory.");
  fail_unless(strlen(c) == 25,
	      "Expected output string to be 25 chars long (plus a NUL byte). strlen(c) == %zu instead.",
	      strlen(c));
  fail_unless(!strncmp(c, str, 25),
	      "Expected output string to be equal to the input string for the first 25 bytes. Instead, they differ.\n"
	      "\tin: %s\n"
	      "\tout: %s",
	      str, c);

  free(c);
}
END_TEST

Suite *strndup_suite()
{
  Suite *s;
  TCase *tc;

  s = suite_create("strndup");

  tc = tcase_create("normal");
  tcase_add_test(tc, strndup_normal);
  suite_add_tcase(s, tc);

  tc = tcase_create("longstring");
  tcase_add_test(tc, strndup_longstring);
  suite_add_tcase(s, tc);

  return s;
}
