/*
 * Copyright 2010 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "suite_strnlen.h"

#include <check.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <strl.h>

START_TEST (strnlen_empty)
{
  const char *src = "";
  size_t len;

  len = strnlen(src, 20);
  fail_unless(len == strlen(src),
	      "Expected return value of %d, was instead %d", strlen(src), len);
}
END_TEST

START_TEST (strnlen_normal)
{
  const char *src = "pizza";
  size_t len;

  len = strnlen(src, 20);
  fail_unless(len == strlen(src),
	      "Expected return value of %d, was instead %d", strlen(src), len);
}
END_TEST

START_TEST(strnlen_long)
{
  const char *src = "Pizza: one of the best foods, especially when topped with bacon and ham.";
  size_t len;

  len = strnlen(src, 20);
  fail_unless(len == 20,
	      "Expected return to be %d, was instead %d.",
	      len, 20);

  len = strnlen(src, strlen(src));
  fail_unless(len == strlen(src),
	      "Expected return value to be %d, was %d instead.",
	      strlen(src), len);
}
END_TEST

Suite *strnlen_suite()
{
  Suite *s;
  TCase *tc;

  s = suite_create("strnlen");

  tc = tcase_create("empty");
  tcase_add_test(tc, strnlen_empty);
  suite_add_tcase(s, tc);

  tc = tcase_create("normal");
  tcase_add_test(tc, strnlen_normal);
  suite_add_tcase(s, tc);

  tc = tcase_create("long");
  tcase_add_test(tc, strnlen_long);
  suite_add_tcase(s, tc);

  return s;
}
