/*
 * Copyright 2010 Nathan Phillip Brink <ohnobinki@ohnopublishing.net>
 *
 * This file is part of libstrl.
 *
 * libstrl is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * libstrl is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with libstrl.  If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "suite_strnlen.h"

#include <check.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <strl.h>

START_TEST (wcsnlen_empty)
{
  const wchar_t *src = L"";
  size_t len;

  len = wcsnlen(src, 20);
  fail_unless(len == wcslen(src),
	      "Expected return value of %d, was instead %d", wcslen(src), len);
}
END_TEST

START_TEST (wcsnlen_normal)
{
  const wchar_t *src = L"pizza";
  size_t len;

  len = wcsnlen(src, 20);
  fail_unless(len == wcslen(src),
	      "Expected return value of %d, was instead %d", wcslen(src), len);
}
END_TEST

START_TEST(wcsnlen_long)
{
  const wchar_t *src = L"Pizza: one of the best foods, especially when topped with bacon and ham.";
  size_t len;

  len = wcsnlen(src, 20);
  fail_unless(len == 20,
	      "Expected return to be %d, was instead %d.",
	      len, 20);

  len = wcsnlen(src, wcslen(src));
  fail_unless(len == wcslen(src),
	      "Expected return value to be %d, was %d instead.",
	      wcslen(src), len);
}
END_TEST

Suite *wcsnlen_suite()
{
  Suite *s;
  TCase *tc;

  s = suite_create("wcsnlen");

  tc = tcase_create("empty");
  tcase_add_test(tc, wcsnlen_empty);
  suite_add_tcase(s, tc);

  tc = tcase_create("normal");
  tcase_add_test(tc, wcsnlen_normal);
  suite_add_tcase(s, tc);

  tc = tcase_create("long");
  tcase_add_test(tc, wcsnlen_long);
  suite_add_tcase(s, tc);

  return s;
}
